/* C++ object-oriented base neural network and derived back
 propagation neural network based on the equations in
 Rummelhart and McClelland's P5rd|/cF ufsrrfhJrcd
 Processing and the shareware back-propagation neura]
 network written in C by Eberhart and Dobbins and
 published in Mfcr0 c0r-Mcopid (Jan./Feb. l990). */



#include <stdio.h>
#include <stdlib.h>.
#include <math.h>
#include <conio.h>
#include <ctype.h>
#include <string.h>


#define ESC 27
#define ITEMS 8
#define ERRORLEVEL 0.02





/* typedefs and prototypes for dynamic storage of arrays */



typedef  float *FLOATPTR;     /* pointer to a real   */
typedef  FLOATPTR VECTOR;     /* a vector: one colum!  */
typedef  FLOATPTR *MATRIX;    /* a matrix:  two colum!   */



/* Prototipos de las funciones que se usan */



void  AllocateVector(VECTOR *vector, int nCols);
void  AllocateColumns(FLOATPTR Matrix[], int nRows, int nCols);
void  AllocateMatrix(MATRIX *pmatrix, int nRows, int nCols);
void  FreeMatrix(MATRIX Matrix, int nRows);

/* Definicion de las matrices de almacenamiento dinamico */

MATRIX  Out0;                 /* input layer */
MATRIX  Out1;                  /* hidden layer  */
MATRIX  Delta1;               /* delta at hidden lay */
MATRIX  Delw1;                /* change in weights inpL */
MATRIX  W1;                    /* weights input:hidden  */
MATRIX  Out2;                 /* output layer  */
MATRIX  Delta2;               /* delta at output layer */
MATRIX  Delw2;                /* change in weights */
MATRIX  W2;                    /* weights hidden:out */
MATRIX  target;         /* target output */
VECTOR  PatternID;            /* identifier for each st */
				/* pattern */


/* main programc  Back_Propagation Network */

void main(int argc, char *argv[])
{

float   Eta = 0.15,       /* default learning rate */
		  Alpha = 0.075;    /* default momentum factor */
int     nReportErrors = 100;/*  Frecuencia de error en fichero */
float   ErrorLevel ;       /*  = ERRORLEVEL; */
char    MonitorError =0 ; /* true when key pressed      */
float   Error;            /* latest sum squared error val(  */
register int H;           /* index hidden layer   */
register int I;           /* index input layer   */
register int J;           /* index output layer */



int nInputNodes,      /* number of input nodes   */
	nHiddenNodes,    /* number of hidden nodes   */
	nOutputNodes,    /* number of output nodes   */
	nIterations,     /* number of iterations   */
	nPatterns,        /* number of patterns    */
	nRuns,            /* nwmber of runs or input line5   */
	P,                /* index pattern number  */
	Q,                /* index iteration number */
	R;		      /* index run number    */


FILE      *fpRun,               /* run file  */
	  *fpPattern,           /* source pattern input */
	  *fpWeights,         /* initial weight file   */
	  *fpWeightsOut,        /* final weight output  */
	  *fpResults,           /* results output file  */
	  *fpError;             /* error output file   */

char  szResults[66];          /* various  file names  */
char  szError[66];
char  szPattern[66];
char  szWeights[66];
char  szWeightsOut[66];
char  *programe = * argv;

/* Read arguments from DOs command line.*/

for (; argc > 1; argc--)
{
	char *arg = *++argv;
	if (*arg != '-')
		 break;
	switch (*++arg)
	{
		case 'e':   sscanf(++arg,  "%d",  &nReportErrors);  break;
		case 'd':   sscanf(++arg,  "%f",  &ErrorLevel); break;
		default:    break;
	}

}

if (argc < 2)
{
	fprintf(stderr, "Usage: %s {-en -df} runfilename\n",programe);
	  fprintf(stderr, " -en => report error every n iterations\n");
	  fprintf(stderr, " -df => done if sum squared error < f\n");
	  exit(1);
}



/* open run file for reading, */

if ((fpRun = fopen(*argv, "r")) == NULL)
{
	fprintf(stderr,"%s: Can't open file %s \n",programe,*argv);
	exit(1);
}


fscanf(fpRun, "%d",&nRuns); /* scan for numbers of runs, */


/* *************************************** */
/* Comienzo del bucle de trabajo Principal */
/* *************************************** */

fscanf(fpRun, "%f",&ErrorLevel); /* Lectura del nivel de error */

/* Comienzo del bucle de trabajo Principal */

for (R= 0; R< nRuns; R++)
{
/* Read and parse the run specification line to obtain */
/* information about this network. */

fscanf(fpRun, "%s %s %s %s %s %d %d %d %d %d %f %f",
	 szResults,                /* output results file */
	 szError,                    /* error output file */
	 szPattern,                /* pattern input file  */
	 szWeights,                /* initial weights file   */
	 szWeightsOut,             /* final weights output filt  */
	 &nPatterns,               /* number of patterns to lea  */
	 &nIterations,             /* number of iterations the data */
	 &nInputNodes,             /* number of input nodes   */
	 &nHiddenNodes,            /* number of hidden nodes  */
	 &nOutputNodes,            /* number Of Output nodeS  */
	 &Eta,                      /* learning rate          */
	 &Alpha);                  /*  momentum factor         */


/* Allocate dynamic storage for nodes and patterns. */


AllocateMatrix(&Out0,  nPatterns,     nInputNodes);
AllocateMatrix(&Out1,  nPatterns,     nHiddenNodes);
AllocateMatrix(&Out2,  nPatterns,     nOutputNodes);
AllocateMatrix(&Delta2,nPatterns,     nOutputNodes);
AllocateMatrix(&Delw2, nOutputNodes,  nHiddenNodes + 1);
AllocateMatrix(&W2,    nOutputNodes,  nHiddenNodes + 1);
AllocateMatrix(&Delta1,nPatterns,     nHiddenNodes);
AllocateMatrix(&Delw1, nHiddenNodes,  nInputNodes + 1);
AllocateMatrix(&W1,    nHiddenNodes,  nInputNodes + 1);
AllocateMatrix(&target,nPatterns,nOutputNodes);
AllocateVector(&PatternID, nPatterns);


 /*  Read the jnitial Weight matrjcesc   */

	if ((fpWeights = fopen(szWeights,"r")) == NULL)
	{
	 fprintf(stderr," %s: Canot open file %s \n",programe,szWeights);
	 exit(1);
	}



 /* Read inputohidden weightsu  */
	for (H = 0;  H < nHiddenNodes;  H++)
	 for (I = 0; I <= nInputNodes; I++)
	 {
	  fscanf(fpWeights, "%f", &W1[H][I]);
	  Delw1[H][I] = 0.0;
	 }


 /* Read hiddenoout weights,  */
	for (J = 0;  J < nOutputNodes;  J++)
	 for (H = 0; H <= nHiddenNodes; H++)
	 {
		fscanf(fpWeights, "%f", &W2[J][H]);
		Delw2[J][H] = 0.0;
	 }

 fclose(fpWeights);



 /* Read in all patterns to be learned, */
	if ((fpPattern = fopen(szPattern, "r")) == NULL)
	{
	 fprintf(stderr," %s: Can't Open fi1e %s \n",programe,szPattern);
	 exit(1);
	}


	 for (P = 0; P < nPatterns; P++)
	 {
	  for (I = 0; I < nInputNodes; I++)
		if (fscanf(fpPattern,"%f", &Out0[P][I])!= 1)
		 goto AllPatternsRead;



	 /* Read in target outputs for input patternsu */
	  for (J = 0; J < nOutputNodes; J++)
	 fscanf(fpPattern, "%f", &target[P][J]);



	/* Read in identifier for eaCh pattern.          */
	 fscanf(fpPattern, "%f ", &PatternID[P]);

 }



 AllPatternsRead:

 fclose(fpPattern);

 if (P < nPatterns)
 {
	fprintf(stderr," %s:  %d out of %d patterns read \n",programe,P,nPatterns);
	nPatterns = P;
 }


/*Open error output file. */

if ((fpError = fopen(szError, "w")) == NULL)
{
	fprintf(stderr," %s:  Can't open file %s \n",programe,szError);
	exit(1);
}



fprintf(stderr, nIterations > 1 ? "Training...\n" : "Testing\n");


/* ITERRATION LOOP */

for (Q = 0; Q < nIterations; Q++)
{
  for (P = 0; P < nPatterns; P++)
  {

/* hidden layer                                       */
/* sum inbut to hidden layer over all inputtweight     */
/* c0mbinati0ns.                                       */

	for (H = 0; H < nHiddenNodes; H++)
	{
		float Sum = W1[H][nInputNodes];  /* Begin with bias. */
	  for (I = 0; I < nInputNodes; I++)
		 Sum   +=   W1[H][I]  *  Out0[P][I];
	  /* Compute output using sigmoid function.*/
	  Out1[P][H] = 1.0 / (1.0 + exp(-Sum));
	}

 /* output layer */

	for (J = 0; J < nOutputNodes; J++)
	{
	  float Sum = W2[J][nHiddenNodes];
	  for (H = 0; H < nHiddenNodes; H++)
		 Sum += W2[J][H] * Out1[P][H];
	  /* Compute output using sigmoid function, */
	  Out2[P][J] = 1.0 / (1.0 + exp(-Sum));

	}



	/* delta output */
	/* Calculate deltas for each output unit for each pattern */
	  for (J = 0;  J < nOutputNodes;  J++)
		Delta2[P][J] = (target[P][J] - Out2[P][J])  *
				Out2[P][J] * (1.0 - Out2[P][J]);


  /* delta hidden  */

	 for (H = 0; H < nHiddenNodes; H++)
	 {
		float Sum = 0.0;
		for (J = 0; J < nOutputNodes; J++)
		  Sum += Delta2[P][J] * W2[J][H];

		/* Compute output using sigmoid function. */

		Delta1[P][H] = Sum * Out1[P][H] * (1.0 - Out1[P][H]);
	 }
  }



  /* Adapt weights hidden:output */

	 for (J = 0; J < nOutputNodes; J++)
	 {
		float Dw;                             /* delta weight */
		float Sum = 0.0;

  /* sum of deltas for each output node for one epoch */
  for (P = 0; P < nPatterns; P++)
	 Sum  +=  Delta2[P][J];

  /* Calculate new bias weight for each output unit, */

  Dw = Eta * Sum + Alpha * Delw2[J][nHiddenNodes];

  W2[J][nHiddenNodes] += Dw;

  Delw2[J][nHiddenNodes] = Dw;              /* delta for b */


/*  Cal calculate  new  weights . */
 for (H = 0; H < nHiddenNodes; H++)
 {
	float Sum = 0.0;
	for (P = 0; P < nPatterns; P++)
	 Sum  +=  Delta2[P][J] * Out1[P][H];
	Dw  =  Eta * Sum  +  Alpha * Delw2[J][H];
	W2[J][H]    +=  Dw;
	Delw2[J][H]  =  Dw;
 }
}





 /* Adapt weights input:hidden. */

 for (H = 0; H < nHiddenNodes; H++)
 {
	float  Dw;  /*  delta  weight */
	float Sum = 0.0;
	for (P = 0; P < nPatterns; P++)
	 Sum  +=  Delta1[P][H];

 /* Calculate new bias weight for each hidden unit. */

 Dw = Eta * Sum + Alpha * Delw1[H][nInputNodes];

 W1[H][nInputNodes]   += Dw;

 Delw1[H][nInputNodes] = Dw;



/*  CalCulate  new  weights. */
for (I = 0; I < nInputNodes; I++)
 {
  float Sum = 0.0;
  for  ( P  =  0; P  <  nPatterns ;  P++)
	 Sum += Delta1[P][H] * Out0[P][I];
  Dw = Eta * Sum + Alpha * Delw1[H][I];

  W1[H][I] += Dw;
  Delw1[H][I] = Dw;

 }
}


 /* Watch for keyboard reguests, */
	if (kbhit())
	 {
	  int c = getch();
	  if ((c = toupper(c)) == 'E')
		 MonitorError++;
	  else if (c == ESC)
		 break;
	 }



 /* sum Scuared error */

	 if (MonitorError || (Q % nReportErrors == 0))
	 {
		for (P = 0, Error = 0.0; P < nPatterns; P++)
		{
		 for (J = 0; J < nOutputNodes; J++)
		 {
			float Temp = target[P][J] - Out2[P][J];
			Error += Temp * Temp;
		 }
		}



	  /* Average error over all patterns */

	  Error /= nPatterns * nOutputNodes;



	  /* Print iteration number and error value  */

	  fprintf(stderr,"Iteration %5d/%-5d Error %f\r",

		  Q, nIterations, Error);

	  MonitorError = 0;



	  if (Q % nReportErrors == 0)

		 fprintf(fpError, "%d %f\n", Q, Error);

	  /* Terminate when error satisfactory, */

	  if (Error < ErrorLevel)

		 break;

	  }

	}

	/* End iterate loop, */



 for (P =  0 , Error = 0.0; P < nPatterns; P++)
 {
	for (J = 0; J < nOutputNodes; J++)
	{
	  float Temp = target[P][J] - Out2[P][J];
	  Error += Temp * Temp;
	}
 }


 /* Average error over all patterns. */

 Error /= nPatterns *nOutputNodes;



 /* Print final iteration number and error value. */

 fprintf(stderr, "Iteration %5d/%-5d Error %f\n", Q,

		  nIterations, Error); /* to screen */

fclose(fpError);



/* Print final weights */

if ((fpWeightsOut = fopen(szWeightsOut,"w")) == NULL)
{
	fprintf(stderr," %s:  Can't write file %s \n",programe,szWeightsOut);
	exit(1);
}



 for (H = 0;  H < nHiddenNodes; H++)
  for (I = 0;  I <= nInputNodes; I++)
	 fprintf(fpWeightsOut, "%g%c", W1[H][I],
				I%ITEMS==ITEMS-1 ? '\n':' ');



 for (J = 0;  J < nOutputNodes;  J++)
  for (H = 0;  H <= nHiddenNodes;  H++)
	 fprintf(fpWeightsOut,  "%g%c", W2[J][H],
				J%ITEMS==ITEMS-1 ? '\n':' ');

 fclose(fpWeightsOut);


 /* Print final activation values. */

 if ((fpResults = fopen(szResults,"w")) == NULL)
 {
	fprintf(stderr," %s:  Can't write file %s \n",programe,szResults);
	fpResults = stderr;
 }

 /* Print final output Vector, */
 for (P = 0; P < nPatterns; P++)
 {
	fprintf(fpResults,"%f ",P);
	for (J = 0; J < nOutputNodes; J++)
	  fprintf(fpResults,"%f ",Out2[P][J]);
	fprintf(fpResults,"%f\n",PatternID[P]);
 }

 fclose(fpResults);



/* Free memory used for matrix. */



  FreeMatrix(Out0, nPatterns);

  FreeMatrix(Out1, nPatterns);

  FreeMatrix(Delta1, nPatterns);

  FreeMatrix(Delw1, nHiddenNodes);

  FreeMatrix(W1, nHiddenNodes);

  FreeMatrix(Out2, nPatterns);

  FreeMatrix(Delta2, nPatterns);

  FreeMatrix(Delw2, nOutputNodes);

  FreeMatrix(W2, nOutputNodes);

  FreeMatrix(target, nPatterns);

  free(PatternID);

 }

 fclose(fpRun);              /* Close run file. */

}







/* Array storage allocation riutines */



  void AllocateVector(VECTOR *Vector, int nCols)

  {

	 if ((*Vector = (VECTOR) calloc(nCols, sizeof(float))) == NULL)

	  {

		 fprintf(stderr, " Sorrt Not enough memory for nodes !\n");

		 exit(1);

	  }

  }



  /* Allocate space for dynamic, two-dimensii */

  void AllocateColumns(FLOATPTR Matrix[], int nRows, int nCols)

  {

	  int i;

	  for (i = 0; i < nRows; i++)

		 AllocateVector(&Matrix[i], nCols);

  }



 void AllocateMatrix(MATRIX *Pmatrix,int nRows, int nCols)

{

	if ((*Pmatrix = (MATRIX) calloc(nRows, sizeof(FLOATPTR))) ==

		 NULL)

	{

		fprintf(stderr, " Sorrt Not enough memory for nodes !\n");

		exit(1);

	}

	AllocateColumns(*Pmatrix, nRows, nCols);

}



/* Free the memory used by the matrix. */



void FreeMatrix(MATRIX Matrix, int nRows)

{

int i;

for (i = 0; i < nRows; i++)

  free(Matrix[i]);

free(Matrix);

}



